@@ -42,6 +42,9 @@ void   bli_arch_set_id( void );
 
 BLIS_EXPORT_BLIS char*  bli_arch_string( arch_t id );
 
+void   bli_arch_set_logging( bool_t dolog );
+bool_t bli_arch_get_logging( void );
+void   bli_arch_log( char*, ... );
 
 #endif
 
