@@ -51,21 +51,21 @@
 void   bli_ind_init( void );
 void   bli_ind_finalize( void );
 
-BLIS_EXPORT_BLIS void   bli_ind_enable( ind_t method );
-BLIS_EXPORT_BLIS void   bli_ind_disable( ind_t method );
-BLIS_EXPORT_BLIS void   bli_ind_disable_all( void );
+BLIS_EXPORT_BLIS void    bli_ind_enable( ind_t method );
+BLIS_EXPORT_BLIS void    bli_ind_disable( ind_t method );
+BLIS_EXPORT_BLIS void    bli_ind_disable_all( void );
 
-BLIS_EXPORT_BLIS void   bli_ind_enable_dt( ind_t method, num_t dt );
-BLIS_EXPORT_BLIS void   bli_ind_disable_dt( ind_t method, num_t dt );
-BLIS_EXPORT_BLIS void   bli_ind_disable_all_dt( num_t dt );
+BLIS_EXPORT_BLIS void    bli_ind_enable_dt( ind_t method, num_t dt );
+BLIS_EXPORT_BLIS void    bli_ind_disable_dt( ind_t method, num_t dt );
+BLIS_EXPORT_BLIS void    bli_ind_disable_all_dt( num_t dt );
 
-BLIS_EXPORT_BLIS void   bli_ind_oper_enable_only( opid_t oper, ind_t method, num_t dt );
+BLIS_EXPORT_BLIS void    bli_ind_oper_enable_only( opid_t oper, ind_t method, num_t dt );
 
-BLIS_EXPORT_BLIS bool_t bli_ind_oper_is_impl( opid_t oper, ind_t method );
+BLIS_EXPORT_BLIS bool_t  bli_ind_oper_is_impl( opid_t oper, ind_t method );
 //bool_t bli_ind_oper_has_avail( opid_t oper, num_t dt );
-BLIS_EXPORT_BLIS void*  bli_ind_oper_get_avail( opid_t oper, num_t dt );
-BLIS_EXPORT_BLIS ind_t  bli_ind_oper_find_avail( opid_t oper, num_t dt );
-BLIS_EXPORT_BLIS char*  bli_ind_oper_get_avail_impl_string( opid_t oper, num_t dt );
+BLIS_EXPORT_BLIS void_fp bli_ind_oper_get_avail( opid_t oper, num_t dt );
+BLIS_EXPORT_BLIS ind_t   bli_ind_oper_find_avail( opid_t oper, num_t dt );
+BLIS_EXPORT_BLIS char*   bli_ind_oper_get_avail_impl_string( opid_t oper, num_t dt );
 
 char*  bli_ind_get_impl_string( ind_t method );
 num_t  bli_ind_map_cdt_to_index( num_t dt );
