@@ -40,7 +40,7 @@
 #undef  GENPROT
 #define GENPROT( opname ) \
 \
-void* PASTEMAC(opname,ind_get_avail)( num_t dt );
+void_fp PASTEMAC(opname,ind_get_avail)( num_t dt );
 /*bool_t PASTEMAC(opname,ind_has_avail)( num_t dt ); */
 
 GENPROT( gemm )
@@ -58,17 +58,17 @@ GENPROT( trsm )
 
 //bool_t bli_l3_ind_oper_is_avail( opid_t oper, ind_t method, num_t dt );
 
-ind_t  bli_l3_ind_oper_find_avail( opid_t oper, num_t dt );
+ind_t   bli_l3_ind_oper_find_avail( opid_t oper, num_t dt );
 
-void   bli_l3_ind_set_enable_dt( ind_t method, num_t dt, bool_t status );
+void    bli_l3_ind_set_enable_dt( ind_t method, num_t dt, bool_t status );
 
-void   bli_l3_ind_oper_enable_only( opid_t oper, ind_t method, num_t dt );
-void   bli_l3_ind_oper_set_enable_all( opid_t oper, num_t dt, bool_t status );
+void    bli_l3_ind_oper_enable_only( opid_t oper, ind_t method, num_t dt );
+void    bli_l3_ind_oper_set_enable_all( opid_t oper, num_t dt, bool_t status );
 
-void   bli_l3_ind_oper_set_enable( opid_t oper, ind_t method, num_t dt, bool_t status );
-bool_t bli_l3_ind_oper_get_enable( opid_t oper, ind_t method, num_t dt );
+void    bli_l3_ind_oper_set_enable( opid_t oper, ind_t method, num_t dt, bool_t status );
+bool_t  bli_l3_ind_oper_get_enable( opid_t oper, ind_t method, num_t dt );
 
-void*  bli_l3_ind_oper_get_func( opid_t oper, ind_t method );
+void_fp bli_l3_ind_oper_get_func( opid_t oper, ind_t method );
 
 
 #endif
