@@ -43,8 +43,8 @@
   #include "blis.h"
 #endif
 
-//#define COL_STORAGE
-#define ROW_STORAGE
+#define COL_STORAGE
+//#define ROW_STORAGE
 
 //#define PRINT
 
@@ -141,13 +141,14 @@ int main( int argc, char** argv )
 
 	printf( "data_%s_%cgemm_%s", THR_STR, dt_ch, STR );
 	printf( "( %2lu, 1:4 ) = [ %4lu %4lu %4lu %7.2f ];\n",
-	        ( unsigned long )(p - p_begin + 1)/p_inc + 1,
+	        ( unsigned long )(p - p_begin)/p_inc + 1,
 	        ( unsigned long )0,
 	        ( unsigned long )0,
 	        ( unsigned long )0, 0.0 );
 
 
-	for ( p = p_begin; p <= p_max; p += p_inc )
+	//for ( p = p_begin; p <= p_max; p += p_inc )
+	for ( p = p_max; p_begin <= p; p -= p_inc )
 	{
 
 		if ( m_input < 0 ) m = p / ( dim_t )abs(m_input);
@@ -183,7 +184,7 @@ int main( int argc, char** argv )
 		bli_setsc(  (1.0/1.0), 0.0, &beta );
 
 		bli_copym( &c, &c_save );
-	
+
 #if 0 //def BLIS
 		bli_ind_disable_all_dt( dt );
 		bli_ind_enable_dt( ind, dt );
@@ -396,7 +397,7 @@ int main( int argc, char** argv )
 
 		printf( "data_%s_%cgemm_%s", THR_STR, dt_ch, STR );
 		printf( "( %2lu, 1:4 ) = [ %4lu %4lu %4lu %7.2f ];\n",
-		        ( unsigned long )(p - p_begin + 1)/p_inc + 1,
+		        ( unsigned long )(p - p_begin)/p_inc + 1,
 		        ( unsigned long )m,
 		        ( unsigned long )k,
 		        ( unsigned long )n, gflops );
