@@ -5,7 +5,7 @@
    libraries.
 
    Copyright (C) 2014, The University of Texas at Austin
-   Copyright (C) 2018, Advanced Micro Devices, Inc.
+   Copyright (C) 2018 - 2019, Advanced Micro Devices, Inc.
 
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are
@@ -309,15 +309,34 @@ void libblis_test_gemmtrsm_ukr_experiment
 #endif
 
 	// Create the packed objects. Use packmr and packnr as the leading
-	// dimensions of ap and bp, respectively.
-	bli_obj_create( datatype, m, k+m, 1, ldap, &ap );
-	bli_obj_create( datatype, k+m, n, ldbp, 1, &bp );
+	// dimensions of ap and bp, respectively. Note that we use the ldims
+	// instead of the matrix dimensions for allocation purposes here.
+	// This is a little hacky and was prompted when trying to support
+	// configurations such as power9 that employ duplication/broadcasting
+	// of elements in one of the packed matrix objects. Thankfully, packm
+	// doesn't care about those dimensions and instead relies on
+	// information taken from the source object. Thus, this is merely
+	// about coaxing bli_obj_create() in allocating enough space for our
+	// purposes.
+	bli_obj_create( datatype, ldap, k+m, 1, ldap, &ap );
+	bli_obj_create( datatype, k+m, ldbp, ldbp, 1, &bp );
+
+	// We overwrite the m dimension of ap and n dimension of bp with
+	// m and n, respectively, so that these objects contain the correct
+	// logical dimensions. Recall that ldap and ldbp were used only to
+	// induce bli_obj_create() to allocate sufficient memory for the
+	// duplication in rare instances where the subconfig uses a gemm
+	// ukernel that duplicates elements in one of the operands.
+	bli_obj_set_length( m, &ap );
+	bli_obj_set_width( n, &bp );
 
 	// Set up the objects for packing. Calling packm_init_pack() does everything
 	// except checkout a memory pool block and save its address to the obj_t's.
 	// However, it does overwrite the buffer field of packed object with that of
-	// the source object. So, we have to save the buffer address that was
-	// allocated.
+	// the source object (as a side-effect of bli_obj_alias_to(); that buffer
+	// field would normally be overwritten yet again by the address from the
+	// memory pool block). So, we have to save the buffer address that was
+	// allocated so we can re-store it to the object afterward.
 	void* buf_ap = bli_obj_buffer( &ap );
 	void* buf_bp = bli_obj_buffer( &bp );
 	bli_packm_init_pack( BLIS_INVERT_DIAG, BLIS_PACKED_ROW_PANELS,
@@ -378,6 +397,28 @@ bli_printm( "ap", &ap, "%5.2f", "" );
 	*perf = ( 2.0 * m * n * k + 1.0 * m * m * n ) / time_min / FLOPS_PER_UNIT_PERF;
 	if ( bli_obj_is_complex( &b ) ) *perf *= 4.0;
 
+	// A hack to support subconfigs such as power9, which duplicate/broadcast
+	// more than one stored element per logical element in the packed copy of
+	// B. We assume that the ratio ldbp/n gives us the duplication factor used
+	// within B while the ratio ldap/m gives us the duplication factor used
+	// within A (not entirely a safe assumption, though I think it holds for
+	// all gemm ukernels currently supported within BLIS). This duplication
+	// factor must be used as the column stride of B (or the row stride of A)
+	// in order for the bli_gemmv() operation (called within the
+	// libblis_test_gemmtrsm_ukr_check()) to operate properly.
+	if ( ldbp / n > 1 )
+	{
+		const dim_t bfac = ldbp / n;
+		bli_obj_set_col_stride( bfac, &b11p );
+		bli_obj_set_col_stride( bfac, &bx1p );
+	}
+	if ( ldap / m > 1 )
+	{
+		const dim_t bfac = ldap / m;
+		bli_obj_set_row_stride( bfac, &a11p );
+		bli_obj_set_row_stride( bfac, &a1xp );
+	}
+
 	// Perform checks.
 	libblis_test_gemmtrsm_ukr_check( params, side, &alpha,
 	                                 &a1xp, &a11p, &bx1p, &b11p, &c11, &c11_save, resid );
@@ -392,6 +433,10 @@ bli_printm( "ap", &ap, "%5.2f", "" );
 	bli_cntl_free( cntl_b, &BLIS_PACKM_SINGLE_THREADED );
 #endif
 
+	// Free the packed objects.
+	bli_obj_free( &ap );
+	bli_obj_free( &bp );
+
 	// Free the test objects.
 	bli_obj_free( &a_big );
 	bli_obj_free( &b );
@@ -465,7 +510,7 @@ void libblis_test_gemmtrsm_ukr_check
 	//
 	// is functioning correctly if
 	//
-	//   normf( v - z )
+	//   normfv( v - z )
 	//
 	// is negligible, where
 	//
