@@ -7,9 +7,12 @@ SDE_VERSION=sde-external-8.16.0-2018-01-30-lin
 SDE_TARBALL=$SDE_VERSION.tar.bz2
 SDE=$SDE_VERSION/sde64
 
-set +x
-curl -s -X POST https://content.dropboxapi.com/2/files/download -H "Authorization: Bearer $DROPBOX_TOKEN" -H "Dropbox-API-Arg: {\"path\": \"/$SDE_TARBALL\"}" > $SDE_TARBALL
-set -x
+curl --verbose --form accept_license=1 --form form_id=intel_licensed_dls_step_1 \
+     --output /dev/null --cookie-jar jar.txt \
+     --location https://software.intel.com/protected-download/267266/144917
+curl --verbose --cookie jar.txt --output $SDE_TARBALL \
+     https://software.intel.com/system/files/managed/2a/1a/$SDE_TARBALL
+
 tar xvf $SDE_TARBALL
 
 make -j2 testsuite-bin
